#include "arduinoadc.h"
#include "ui_arduinoadc.h"
#include "poulabout.h"
#include "arduinoplot.h"

#include <QMessageBox>
#include <QStatusBar>

#include <qwt_plot.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_curve.h>

ArduinoADC::ArduinoADC(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::ArduinoADC)
{
    ui->setupUi(this);

    trayIcon = new QSystemTrayIcon(QIcon(":/poul_icon.png"), this);
    trayIcon->show();

    trayMenu = new QMenu(this);
    trayMenu->addAction(ui->actionConnect);
    trayMenu->addAction(ui->actionDisconnect);
    trayMenu->addSeparator();
    trayMenu->addAction(ui->actionAbout);
    trayMenu->addAction(ui->actionExit);

    trayIcon->setContextMenu(trayMenu);

    portName = "/dev/ttyUSB0";
    serialPort = new QextSerialPort(QextSerialPort::EventDriven, this);
    serialPort->setQueryMode(QextSerialPort::EventDriven);
    serialPort->setPortName(portName);
    serialPort->setBaudRate(BAUD9600);
    serialPort->setFlowControl(FLOW_OFF);
    serialPort->setParity(PAR_NONE);
    serialPort->setDataBits(DATA_8);
    serialPort->setStopBits(STOP_2);

    connect(serialPort, SIGNAL(readyRead()), this, SLOT(onReadyRead()));

    ui->centralWidget->setEnabled(false);

    ui->menuBar->hide();
}

ArduinoADC::~ArduinoADC()
{
    delete ui;
}

void ArduinoADC::onReadyRead()
{
    if(serialPort->canReadLine()) {

        bool ok;
        QString str = serialPort->readLine().trimmed();

        int number = str.toInt(&ok);
        if(ok) {
            ui->plot->addValue((double)number/1023*5);
        }
    }
}

void ArduinoADC::on_actionAbout_triggered()
{
    POuLAbout dialog(this);
    dialog.exec();
}

void ArduinoADC::on_actionExit_triggered()
{
    ui->plot->stop();
    this->close();
}

void ArduinoADC::on_actionConnect_triggered()
{
    if(!serialPort->isOpen()) {

        if (serialPort->open(QIODevice::ReadWrite)) {
            ui->statusBar->showMessage(tr("Arduino collegato"), 5000);

            ui->centralWidget->setEnabled(true);
            ui->actionConnect->setEnabled(false);
            ui->actionDisconnect->setEnabled(true);
            ui->plot->start();
        } else {
            QMessageBox::critical(this, tr("Errore"), tr("Impossibile aprire la porta: ") + portName);
        }
    }
}

void ArduinoADC::on_actionDisconnect_triggered()
{
    if(serialPort->isOpen()) {

        serialPort->close();
        ui->statusBar->showMessage(tr("Arduino scollegato"), 5000);

        ui->centralWidget->setEnabled(false);
        ui->actionConnect->setEnabled(true);
        ui->actionDisconnect->setEnabled(false);
        ui->plot->stop();
    }
}
