#include <QTimer>

#include <qwt_painter.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_layout.h>
#include <qwt_scale_widget.h>
#include <qwt_scale_draw.h>

#include "arduinoplot.h"

ArduinoPlot::ArduinoPlot(QWidget *parent):
    QwtPlot(parent),
    interval(10),
    numPoints(100),
    refreshPeriod(50),
    newValue(0)
{
    setTitle("Arduino ADC");
    setCanvasBackground(Qt::white);

    plotLayout()->setAlignCanvasToScales(true);
    canvas()->setFrameStyle(QFrame::Box|QFrame::Plain);
    canvas()->setLineWidth(1);
    axisWidget(QwtPlot::xBottom)->setMargin(0);
    axisWidget(QwtPlot::yLeft)->setMargin(0);
    axisWidget(QwtPlot::yRight)->setMargin(0);
    axisScaleDraw(QwtPlot::xBottom)->enableComponent(QwtAbstractScaleDraw::Backbone, false);
    axisScaleDraw(QwtPlot::yLeft)->enableComponent(QwtAbstractScaleDraw::Backbone, false);
    axisScaleDraw(QwtPlot::yRight)->enableComponent(QwtAbstractScaleDraw::Backbone, false);

    QPen gridPen;
    gridPen.setStyle(Qt::DashLine);
    gridPen.setColor(Qt::lightGray);
    grid = new QwtPlotGrid();
    grid->setPen(gridPen);
    grid->attach(this);

    QPen curvePen;
    curvePen.setColor(Qt::darkBlue);
    curvePen.setWidth(2);
    curve = new QwtPlotCurve("ADC1");
    curve->setPen(curvePen);
    curve->setRenderHint(QwtPlotCurve::RenderAntialiased);
    curve->attach(this);

    enableAxis(QwtPlot::xBottom, false);

    setAxisTitle(QwtPlot::yLeft, "voltage [V]");
    setAxisScale(QwtPlot::yLeft, 0, 5, 0.5);

    enableAxis(QwtPlot::yRight);
    setAxisTitle(QwtPlot::yRight, "ADC value []");
    setAxisScale(QwtPlot::yRight, 0, 1023);

    x.resize(numPoints);
    data.resize(numPoints);

    clearData();

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(timerTimeout()));
}

ArduinoPlot::~ArduinoPlot()
{
    delete grid;
    delete curve;
}

void ArduinoPlot::clearData()
{
    for (int i = 0; i < numPoints; i++) {
        x[numPoints-1-i] = i;
        data[i] = 0;
    }
}

void ArduinoPlot::timerTimeout()
{
    data.pop_back();
    data.push_front(newValue);

    for(int j = 0; j < numPoints; j++)
        x[j]++;

    setAxisScale(QwtPlot::xBottom, x.last(), x.first());
    curve->setSamples(x, data);

    replot();
}

void ArduinoPlot::addValue(double value)
{
    newValue = value;
}

void ArduinoPlot::start()
{
    clearData();
    timer->start(refreshPeriod);
}

void ArduinoPlot::stop()
{
    timer->stop();
}
